'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.serialize = serialize;
exports.deserialize = deserialize;

var _constants = require('./constants');

function serialize(item) {
  if (item instanceof Error) {
    // Since we cannot send the error object, we send necessary info to recreate it
    return {
      type: _constants.TYPE_ERROR,
      name: item.constructor.name,
      message: item.message,
      stack: item.stack
    };
  }

  return item;
}

function deserialize(item) {
  if (item != null && item.type === _constants.TYPE_ERROR) {
    // Try to get the global object
    var g =
    // DOM environment in browsers
    typeof window !== 'undefined' ? window : // Web worker environment
    typeof self !== 'undefined' ? self : //Node environment
    typeof global !== 'undefined' ? // eslint-disable-next-line no-undef
    global : null;

    var name = item.name,
        message = item.message,
        stack = item.stack;

    // If the error was for current action, reject the promise
    // Try to preserve the error constructor, e.g. TypeError

    var ErrorConstructor = g && g[name] ? g[name] : Error;

    var error = new ErrorConstructor(message);

    // Preserve the error stack
    error.stack = stack;

    return error;
  }

  return item;
}
//# sourceMappingURL=transfer.js.map